
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: ipairs, IsAddOnLoaded, pairs, table, WeakAuras, WeakAurasSaved

local coroutine = coroutine
local next = next
local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration WeakAuras")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.LINK_COLOR = CreateColor(0.5333, 0, 1, 1)

M.RegionType = {
	AuraBar = "aurabar",
	SubBorder = "subborder",
	Icon = "icon",
}

M.RegionTypeWithStyle = {
	[M.RegionType.AuraBar] = true,
	[M.RegionType.Icon] = true,
}

M.isAncestorIgnored = {}

M.originalTexture = {}

M.regionRegistry = {}

function M.GetColorForBorder(border)
	if border and border:IsShown() then
		local r, g, b, a = border:GetBackdropBorderColor()

		if (r ~= 0 or g ~= 0 or b ~= 0) and (r ~= 1 or g ~= 1 or b ~= 1) and a ~= 0 then
			return r, g, b
		end
	end
end

function M:OnInitialize()
	self:CreateHooks()
end

function M:OnEnable()
	if not IsAddOnLoaded("WeakAuras") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	self:CreateHooks()
	self:RegisterRegions()
	self:RegisterEvent("LOADING_SCREEN_DISABLED")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_LAYOUT_CHANGED")
end

function M:OnDisable()
	self:UnregisterRegions()
end

function M:CreateHooks()
	if WeakAuras and WeakAuras.GetRegion and not self:IsHooked(WeakAuras, "GetRegion") then
		self:RawHook(WeakAuras, "GetRegion", "WeakAuras_GetRegion")
	end
end

-- Iterate over known and discoverable WeakAuras regions.
function M:IterateRegions()
	return coroutine.wrap(function()
		for region in next, self.regionRegistry do
			if region and region.Show and region.Hide then
				coroutine.yield(region)
			end
		end
	end)
end

-- Iterate over sub-regions of a given type.
function M:IterateSubRegions(subRegions, subRegionType)
	local subRegionIndex = 0

	return coroutine.wrap(function()
		for _, subRegion in next, subRegions do
			if not subRegionType or subRegionType == subRegion.type then
				subRegionIndex = subRegionIndex + 1
				coroutine.yield(subRegion, subRegionIndex)
			end
		end
	end)
end

-- Register the given region for updates and style it.
function M:RegisterRegion(region)
	if not self:IsHooked(region, "OnShow") then
		self:HookScript(region, "OnShow", "Region_OnShow")
	end

	if not self:IsHooked(region, "OnHide") then
		self:HookScript(region, "OnHide", "Region_OnHide")
	end

	if region:IsShown() then
		self:Region_OnShow(region)
	else
		self:Region_OnHide(region)
	end
end

function M:RegisterRegions()
	for region in self:IterateRegions() do
		self:RegisterRegion(region)
	end
end

-- Unregister the given region from updates and reset it.
function M:UnregisterRegion(region)
	self:Unhook(region, "OnShow")
	self:Unhook(region, "OnHide")
	self:ResetRegion(region)
end

function M:UnregisterRegions()
	for region in self:IterateRegions() do
		self:UnregisterRegion(region)
	end
end

function M:ReregisterRegions()
	self:UnregisterRegions()
	self:RegisterRegions()
end

function M:StyleRegion(region)
	if (not self.RegionTypeWithStyle[region.regionType]) then
		return
	end

	if not self:IsEnabled() then
		return
	end

	if self:IsAncestorIgnored(region.id) then
		return
	end

	B:CreateBorder(region, C.BORDER_PADDING)

	if region.regionType == self.RegionType.Icon then
		self:StyleIcon(region.icon)
	end

	if region.regionType == self.RegionType.AuraBar then
		self:StyleBar(region.bar)
	end

	self:StyleSubRegions(region.subRegions)
end

function M:ResetRegion(region)
	B:RemoveBorder(region)

	if region.regionType == self.RegionType.Icon then
		self:ResetIcon(region.icon)
	end

	if region.regionType == self.RegionType.AuraBar then
		self:ResetBar(region.bar)
	end

	self:ResetSubRegions(region.subRegions)
end

function M:StyleIcon(icon)
	if (not icon) then return end
	
	if not self:IsHooked(icon, "SetVertexColor") then
		self:RawHook(icon, "SetVertexColor", "Icon_SetVertexColor", true)
		self:Icon_SetVertexColor(icon, icon:GetVertexColor())
	end
end

function M:ResetIcon(icon)
	if (not icon) then return end
	self:Unhook(icon, "SetVertexColor")
end

function M:StyleBar(bar)
	if (not bar) then return end
	self:StyleTexture(bar.bg, nil)
	self:StyleTexture(bar.fg, [[Interface\AddOns\SixUI\Resources\Textures\Light.tga]])
end

function M:ResetBar(bar)
	if (not bar) then return end
	self:ResetTexture(bar.bg)
	self:ResetTexture(bar.fg)
end

function M:StyleTexture(object, texture)
	if (not object) then return end

	if not self.originalTexture[object] then
		self.originalTexture[object] = {object:GetTexture()}
	end

	object:SetTexture(texture)
end

function M:ResetTexture(object)
	if (not object) then return end
	local texture = self.originalTexture[object]
	if (not texture) then return end
	object:SetTexture(unpack(texture))
	self.originalTexture[object] = nil
end

function M:StyleSubRegions(subRegions)
	if (not subRegions) then return end

	for border, borderIndex in self:IterateSubRegions(subRegions, self.RegionType.SubBorder) do
		border:SetAlpha(0)

		if borderIndex == 1 then
			if not self:IsHooked(border, "SetBorderColor") then
				self:SecureHook(border, "SetBorderColor", "Border_SetBorderColor")
			end

			if not self:IsHooked(border, "SetVisible") then
				self:SecureHook(border, "SetVisible", "Border_SetBorderColor")
			end

			self:Border_SetBorderColor(border)
		end
	end
end

function M:ResetSubRegions(subRegions)
	if (not subRegions) then return end

	for border in self:IterateSubRegions(subRegions, self.RegionType.SubBorder) do
		border:SetAlpha(1)
		self:Unhook(border, "SetBorderColor")
		self:Unhook(border, "SetVisible")
	end
end

function M:PrintImportReminder()
	if WeakAuras and WeakAuras.GetData and WeakAuras.GetData("SixUI") == nil then
		A.PrintF(
			"Import SixUI's WeakAuras pack for '%s', %s', and other enhancements: %s",
			A.Highlight("Resource Bars"),
			A.Highlight("Vigor Bars"),
			A.Highlight("|HSixWA|h[WeakAuras: SixUI]|h", self.LINK_COLOR)
		)
	end
end

-- Import SixUI's WeakAuras pack.
function M:Import()
	WeakAuras.Import("!WA:2!T37c8XXv99I7z3yC8cjww(rCCijlchhToYksRSKLvW2rR0kP1rV8URKsSJRMz3D0ot8QzNmZUYsbmTrnackHhIsGau(7QlnLBFeO6gUKp9sdTU5sluOCq8OdukpCjPa95)qBbAVT()VFNZ8A3D2vkkUCL5F8hlPDpVF993RZVZ5Wnv9txFM61J3EZD28HRpZc7tBUrMAkDXcCAZiQPlNxX)YYktLxBAHcWxc8cckPLYR1NMW0IjNtvmvIEIhn6WlkOiZsrhrMwqwPVifGiJOKxrCLmf1OXmjgKMUy68kz0NxuqNwaKaxe)yIcAIkzliTL46fe0k0hzRKaKxj5vrUo4Vx)wsnLSISUufbRNVOwAXuYtRMxRq4IA5uVjPcfu176oVZZjKnFZY5VZ8D3sB33OfLM4oBvTEOUlOLpxoXm9ijNlduNNsTUi5lui)0bhuCQcbJiOPVYqY6PfZLtqrmFr9lfxKvlbJb5wFf7VIjDPXLZMxJ(XLtLxlJOwp5ZLx7eCCC(xIfq0mzfv3EIhSOGMyW(kMlxWjKKliMsxCAyeoERn3sZTSKEXuXfZcds6b0esJJw6Deb6Yfcq2wasDbwQq(0MZh7H7Tizw0jKFiXnVywT8fvX22E21oEYvsjK(Sz0YRYAh(97Fu)T0C7VGSsbrnfHCJZkKhnfReKwMnFoAEibP6j6WjJg)ISOyld26YtHt1W0Jqbb)AZAU4ifo6gR3fThCxcNvLPn8aPGppLC2axIvqrmBrWaDo5h6He0Yemz(85kiRAoKftbks)lPrhaW1eXPDPi5YlKPVufeYjQuy3XNUiKLaXtNtqxF3KJeiIo07zFqvmn(Hvekwa6lJOYAgZxqtoBwO)U39VyrfZVeqZ8dV55HSLlxSm6biBnUaSgnCKIWaEk1CcZbZlIZa1AQbefYvqAPmIPko1uyttBGOdoAFJn4LGzmAssarip7Yt2t3jsozIKDhpPDuJQjcrfpXOrhCW4kWOOEG46PHUJ)gKZepH8SJfBrDXCtrh8jhiCr5mlpwRHkCKtk1sJJ3s9ZCnNV(0x4Gl0fX3bFbDyb(aZLstotsHu5ehFLergmsxbpzrHZkRKvAPHgc(2ac6feLuVrwC9KF6ucfcclylQPjsxvjbRePXfxuiZCb7rsm9zLUelOjGHJG9jMrICn7xDpYzvYRjAowgf7or10YRPZVmqmiTineD(uQWQAOXpczZZpwSr7ooScISLoixB5ivY2QiK6kpK45KNwUqiYOlfzKKjhzObJ2xsYoi7CrbT0dsPo8i7MSRtrgFzwdM2BjjjtqU35lQloiMBjYbpy8ZjNPG0a390)s(20M20hNCtu4i5vR1AWrLNvmhPzYDs87xJrZsKNC0aKtUCStMjv2hiMuOIHNplomdZ2neVxYRfbClc1KA3kzZjERPKeLZkvyG7w)JD5)JlF5VpP1aAanlCnfhzFBoUUQqAXTcZWKDt2oPEs7xmZCWKVCA6IAY(TXKKB)rjnkrUfYTsg2pjmPT51YFUeyU9tUdUfX2Weyh5vcHxGsbLlcUkagRAAlP0eYixu)rEvKdXrAja510bPbtAfKqKFHLMci00JSw6CI8lNoFUItRywYh(ZSugzyjBAaonhIAi3f(BTPyKDOH0fneyzf2IPqmGOefRP9qafDaZ4J86yHNphlCgknmn7DI)Er1zunHUW3IlKdwqfHoJESH3p54bi39tq6H0lPFYalHGRes5p3ikxuN(NUH2YmIKUdCjyEDsHCNtyo9cAffLirxQhBcnWYNEiXiNGmuaY9qgC5mY6pqrfAwdlKlN62fOFojdWpu(mI)oB6sin8cckf6dImVwQ4D3BSXsqUnTwBRZwdFKdfb6ZNRHXgDP0Iift6IGbJVW9SOmKXj1LZisEda5aDHcixnObDAyAEZKxXlCwrybcsjQqCCUIhN7VKEb50NDUEnzakPvqCwiFIXtKpNCMiqXK2hB9fj1H2(kWkhTZg3AU(s0VgbgbZGn9WD3BVltdAa5mzevIpC0XJgxD7LKjmHr6ESKJmFkbtordCCsD1fCtBAFdC8V6U21TE5l)hnWXFUpXN4tF5l)D9V4ua7W(YjKvxBKXsoySHJUOv(cdlt58hbOOMNpc235jTS39hpnsWOVgYR2Wrpk5U9p)mcAYi5iT0scAzf1JpJqUII13c(Df477D)ZRcK(f1kmh59rUaTqFblwuXu6tdAasRKgyhKj)5usCozvrEyHhYQdgBp8NHOVFy1tkDze6LA65kijNMCxyyKNio0Zlm3kY05Z0IuXlGy67M53o)TY)A4jpX2zRKzlixbxkrxrJSnLiNZCn8Yy4MR1LiZgqD7SaCvQsKIqbVFafiNHN0buYKFxy1LJKqJsPckro27awAV)vYkQiQjNMTK(sSL09y2hJdCstFwYhKC3lSx2QBqigTu4mhpcgyvFjLap53J8bxMMp2WSp1DsBJus0jq2y3Jcu080UOMikagYtaWfiDXjr2A8iczElrXQVvAV2CstcbI08szjom0H4PXJT8SPZiPUd8BIZcqgxjbGNAfvOC6GUZhJ8Xxrd4wMWk(RD0R5fWS5QDdWx1B0AGi4OA5ZQjQRhSrAEcbSHPfocPxu3vXSODXWdsWvuhewZetpy(SYPvVRPOO(8knwiuGGW)0ebuMsWglC6wptqq8ScNo8zcfuqjd8P2otaarfW6FKF5FNnrE4WckZTefjzl2KFIpoGri5(KiNs6I04yY(CVCQ1bnOUZ8aqlrmJyMHKvKw388iJUmJzx8y9pqYvSeiIbmUOfWaLHuAjegMGk7jh5GCKGNA3KTUciSbINWLVkzmda4qlYcygtH5MmTTyPMcDnPiuMKB2ugTjNrwxgWX8KTUiwI0anBntYwOhginyfsE6iHDrHsI5FHDREJi9TjP)IPXWKkMFGeqDh0WrckOIcPPS2I0n81LOS3etcrhz4rgo6kS67Enf1CfZYtPasmoYWadi16PHPljado3hlDV6n9cSecC4z8nF4lsdWQNTOD5CXUbIw5coSaiqZ5CkEKSWnO(AQOp0AZM4e7(YLyvVTOBpgRqohmAmHMG68ty(HfTllvVeglyJ3MDzhYSxB2BG1yU6Io6uygSlH2xk2Wdhn(KiN1LOXL2zf8w1G5sA1xVRSzoWU1sgfnd0pBmdxAlp1CKdSWn5XaI6KfKNwCsvOBit1q0JjEvZpehtAgYxN8ni)LKVj5VItnu1kptXKMSGeqvqcw4VfY3I8TjFNx9MiFFYpqDFvlF5eZkKEUjNkx(8A8KV7dtUeVAWQL4PZNzsnQqF)1KVh55UbYZdWW)gYJPElvlhSVXr(HnCBQK)ooYFptAY)HLzd8uCl5FYCaN8)l5fgCtKFKFY)m5aZBbF5lr8mP2u3rwu8nGW5iA4VP0KxAGrIh7uJmCYUh0n5M7JZuC140aLkN4JWSsQ1dbzrwfyKeN2)OcAYHYp)coeiSKj4P25obrb(jdC8LRVEq4GGdC8pbv4GNBGJ)32wBDUPnDlGeX90zVPLh6bAlvKXazkpaidNQKavsv1DA11quBVY6Omaz4VKOc(H(n7CsrEO85NgL1TTLmjBdPEwUDztVMrUwEkq3dHZ2nWCsV5y6M6CmIQiKGGfKevOPYfH9WTqdrmNUODuqHGkXnPOYmnRJ0tO07lnOMfNvvMXbmjm1wAzJ)lxEq9SG2CqdE0vPaoyW(flGFYSNuw7KUKSrNI72GwUtcr2qw)f(beQhuLafIf1aLQpVVD8Uq9igNs53nW(2F04oFZuQqQwjhD58oROixMQCr7AyNaOERwNZSWyQzG(tQI0)Wy9rjAUlqzi2YVKmzvvBmgkimOEG493dOWf0pIiOD)Jf7GoF7GjWmSWqK3k5nqEBsgC7YGJkvm5xHkhm5TlrEuY7G2ASf3L8U4iVBYIK3d5xL8Ejpg59vtrvjVFYJt(au5hjFqEYhIhv65)Me5JWt(nabf)AeDhDzifyAXaYST7uJgpw0ejbr2m4UbsrAeZq)nk)hMIndXTzYSuDwqbnqjA0KiZ5i5i5HO)(jHo21kr(tQq6aYFkh5ZYeby8uavra1grxvwbzV7MB)2g3G7gNxhe9xSqE1YJTomwdUBQ8WhL85i)zsKpVe5laucT47t(ZXF9fNHqyKJ(sKBMScW9M8LjFfYxL818t(laoYedOaCO9AtpLsKSmkGirwKiNIBICK)XsiTTvtkBlCtK)f)K)vYp2DH)tkHEn5NAslM8Vvrn9Vxj1uY)hVPHcqcY)bVbNFEGeyjKr)pjx2GBtmQJarodooy(XNeq3QOys1rd)qXos8Hn42cVTcNlStQuTfygFscL2MkHkZ6o9Ggws6IiwHk8PCg93wKWDgU94T2slD0s82c3ANHJ3zlT1EBXpc872t1AhTCO2dNQTd3ERD0zQ2Ga7SdYqNArkPHja8M2aIA5L1NwTUUvsJWYGda6Xcc2lSyeG0qMCaCC5HfbYqANtwjJo2AIIszNGzAk(LqjW7jhiYJopkKUgKauOyjuZH(rnwaQcfrnpop5nsIs6BrS5Zm1eiE(Ym8(KsOAR)yq13PtjRioKqbqnoDAzpy(ZjQxagZJqvKg0ryLPzrZu9ak1hMmpkKSb31S24BW47yWvVb3oqEfgCVc)gCbm4ELiTmyQCRuocaBqKI3T)OgCVkFi7fdURJUa4s6tNpFbjlTdGj1RNgUb32oQbxDarT2n42(YcMSabgG13gq0YGBN7I8o9H6JVqVMeISuo3nnO3PVRGKGoW(j)ADqUByf)h2CSMQGm5cQpNfpUGMSgySwyl1AkyAG2zfCx0b1vAQIWafwW8BXPKvcuMB0cb)alSYzozLKAY1ZKtLxP7yExULWZ6xhgbwkUokAb5cDyW1L6HCyx0DMmJOOF)282V)HeZilC)jWuRF)rKpR4a51uAoF2SlrlbQU2rgf4(Tc77OreueZLAi0qFAqnXSHaqWOdwndvOz1ExLuT5lPAhfwKRr)yjna98kcAyTBW96m4G1whZG742vcmlNkNSYzbXAOZX(jF4iiIHCb)iFhKPZc1HmrqluS7fHMAUCIkzfPCA6rtaTkSiWu6jUzg7idU9izAoc0yc7MzgHoymJO0LmZcpZSeMmGM1fBO7IMVpkMVpk5)oe0U3oVnVjYXEsgvUpdZSAAd0D8H6BSbn4UtacDlsa5adUdYJ2kdOtyH4n4U19Mk8HA5qh6Wi43IojsACECHiDlE6nAKX6RV5ZaukgtrUau6dbzSrdUqgCaS9oUiJiklwE63SYl10cKFFPLTTUaqvJ80neUfkfR)aGds3b0mJyVgC9JTcQr(OAlb63dkKMM(LCWNJTYKDpw8UNS7rhDWyr7LCX9QwFI5ue1YkJMAliqs8CfGUByYpM1vMH8jPeojrJGIRG9lYY8ubMnn8cAHFMXxqRj(SQ7Zb4wiKf8WTHeW1(mRgKIb5RTebvAuaKpFKPeasYiNE4lLlcWeveHPmaaBoYxG7fNma7QgYayYwg5wJSLbE1)qnZnuyTieWE9qiaRshegGjaaW93rudRA6fJaaaFcxS6TzZBXMznYvYLKdoSMySDqwtOKD2CMAfnWoLldWBcec)(ySWySNOcXICT22rr(zjzSKq(pOqVuUqNC54rp0ek5KJ3MyiqSLF2XpkGfPPPmjIWKh(32MYd879t(DP2WStBchxFjeoaecS2BCg1HXNL73XwBnGcxckpPMcsT6xtbNSPGwMu0wvoAubp6rd2WiJMm2idNOHs54yxkNUHgoJlEGD1JMiemnUgBR9MCuTk4DeST2D0BQCEx4gdyRpyjnG4r7U37BYEgiAp3Z6Pry11kTPuAhED3GMSVydhlXar7TSwwbud6MrdH7mEVwQktgYa5t2MpQR(ACvBnf0RQMYjyiBXer2VltL6GPwiqAM8RbPbj2ToOX5fHSnau4w21wF(ZJu52pf1EmBXiPlHgsEwzfyD(RNUybnc7aGobqa4cOMObY2XMmLeM1ADiWwyHuMmI2HRAkUoY0hcTbQz)AGfhk2KDXE(abS5WUkOUsRRsmTpld9PLFAAptVr0ySU6Qnf817UXxTYeIQSaoFOGioAXm5NKHk4jBB)ZdFJkrkmP0Lb3D)IMjsQoATZ2ATZAWhzcp5J8qtCQELFWIJR2zcKpsF)mNpYZxn7QyWDdv3MkhGhTPcvIxQ8Rlr3SjQuCEByfYX(W1IhKVVHJqzvYeY0HogU402F2nzZRuSKCZdO1wwpu)tgpw)9hnEzvotXqMjlpAznfqKZx)5llLU7IuuLREShzYTbsz2WKchPBgMRSYcTcdIY6toMSK1LQmJvuCEOlAjXBdoROkxttg1QUoyvQPsN2kzSXRPVkSW8QZDETvMLz2xGJTb3Px0CXrxHnzkBWDgdUFbtwWmWXxZG7MLU4uIfslz6av8Q7Y0AwJJ77(K4oTluiVw9hJUvY17osaFKsudO)06bI0x(8zUi(RG7pyVAGwUAMP0NADMFYUSA4OGQX3jpvJraCEr3LPp6UrmP7GKaDOE1vvhkQUt(jpmcU)VArko5YQ9wi0dntXw6vk8pJeUyElNOAdVKf3(JATdrOP1wBYxWyZ5W0dz39k87sQJR9kTuhTEfx8IxIcvSgLFWGtaeCOdM4clbIpmn04fYkcdyPKi1bJwfHpkylUGbxhsSnaIPY5DBk5W1zA6tQWgGedwIlCVuXfm4oSTTpn30fqo6fUnetbleUMaGGcJackC)RF3t4KlF4CQZn0HMks4OHOiODDc1DMyOUcgro7eYz1dID7CYkIQ7fdTFn5mHdoimJWCDYG0DCu91IXnsQhqK6rwbtIoZHOwqC7)piKICcQ6IQ7atusC7WlaRiO2ax3259aWjkXuWsD1nMip7BZKBJ8MR016GfPK7yRKM8JyECFcabBa5YW1VhYpPDshKd3NPKiD5smhMQZhna5y7Lr49nJKHnP)rjpJcOyAVC0GAdJo0j5gGb)dGIKTW(TMa29Yu3kITtUOyB3WlTzduEp6CqxArs2f62XQ1JFaNd0c24Rdbm6Hwed6KfflkUMh92JVx13VQJFWGhmeIJFNC5e9DKcrs0Zj64izlD0eK)8kYGjmsAWDo6KZW(Xr0f6K8wJZ2LDqYxCnng)vKHruUzGLdUVA(azBp4NQEsk)MJkMsOdn8pR1(kLS3h6bhyW0tPxuN5Px29MucuBoZ6sR4gzOv(ALN1hZ4HKpJdDZRThh6MjVVbJo5WDpuuKmhZv)AiqauOR5qHoS3CaHzeZek4BePfIYK4sOYstds(98mxEQscJGmTWAOefMlhDBLASeYHi1VUIPB2BgmVqgXmnYCJQgB0m0MZKclemZu)ETXgmJyYrZvmlqdDs014BietO0qnd0XNsoNO1xtzw543vKZvTMzfnc3ntL8fO)y1KOnqNMxzFTz3Tw3X19OXk7Rn7A0P8OqN1h3tcR4dvJMUzsSYc0(1TgWxRJ3o1S7PmxLdUqOgdGJOGmEDC3ckJn6YJMTlJsLeN2ys4UU8ilHCliR31lUnILSdyo13rzvI3RbRzl9nYYQBWsfn(kY0rDQ6aULdpGD3T8jvZMI3dfo1DvN7Rs5z2RQYunUu2kApw5uwvxA)21YKkQ7YYztMcLvIMxQYMhpeuYRMQi(zQvKDRQMBogw5ObDK)hq4Hcc)Ql2cqCuVY8sBK5vvPuR8iVSfrENxy5tcxZXU22v34k3lvkPyo)QUgoHNaU1WmznMfDWTEykaGM1rd(IK(QNQ7Bs8di6bfBo0jW0RiGM7twt0kulQZvOtUtCoKSlhp5kT2LExyP3ydJOmkl39WK2VHxQQHRc6SwOXgCn10vWOYOFua4hOf0ujTIM8OhdmCC6un4HtzvMVL5gww2CDfJwE16ggYFoAoCxzRL(fMtn71onuv)hJPYbkpH62RGamnY6QisgOKgjQ5HLkIuvl4KO2qmyxRrvLPwBenO51GPHjg57d3SEoQXmzgy0ulf0OKVli8TYHk0qAAZKpKevJhqMsBdKcATsKE2RJAhYXlz)SC5v)uJmgkfZhQT1i100cGYquXjFdlCauIC3IpUCIOdgTNK9fhOiqfKCH6trDPUU9x)SCRy6X5k65qFPS(5(8xhuUCSu0VF2FJB(3i(trpvxZ6N80n0Al0JLi(958NI5zZCRyxu9bKxupV9oQzPwBtbPN6X718V3hSIvmxbH718V3xPYjqtAW7iOD2dEaZ0hYQa8k27JXOIzzvtPHDAzOM1QBxh(o6hHjTc2l5PDBWfxr9RirfS2Xr1OACCc6YhVSJYJt(oCwMrbnub14ggC)sudPuMFOT0iJL00lNDzMdp87yhN0M8J4ip3nVWn6PrwO19pvI8DhDZTDKMBRnxMzH2uIhf9spYFd8zVS1YbCBRLFYwyv1uUrhi0aqimKX)HerkMPEfuaMLhn5N3GBluBXekCYrgfv6ivm16CDYsO)sD72oZsxM69zVpdOEmnTf0Q9NC5qZ1yYbI2tMHMBckqcqcFeezVgGqw6I4dTkHP6HvUXW)pzNlK05ONcm8eGSOT3TIl9HLwZtIAW9lt12HD0SuR3ojo2u81DuQwqhLPU35b19(qKrn4EllCeIVtssS7suR9yOsTbdlrwOHbJBIFn4wy9R)3zEL0kJo2njEMc3R62DxFuTOjW4aSi2Lg0m)WMKrIDGbp7GjN(CT06dm0W6KhH8MmNFbvSNcp2GWOE2Bf1TwMJ2hFJKhyRu8XBPHXg1u)Afggb1Gg01f00E6xjjNFs(ff0tdqv0nmAcTtXzjQpYRcv3wRuxGeOC)(rDVRHI3VEyT6A1mgpiSA8Xn4(aUneW1eyHqLq78ns34P19q))vsKWG796cxAW9yVejcyW9(UcsbWlewTjjavcLGGNy9tfn8eDmXjg)SdCitdXaqDGkZQc1PKf8cABW9H4PEj2Vw1GYgCF46pMBG7fyaxYbw4OWIeT8flGwcegyXLl7MCFsQbn9N9Kv9GvOU)QNe3hTdPxYl8qZZ95LOn8zHr25ToVoWsQupmD9iU08GCRu6rIHoSFYLZMRT5o1i5p5y3Z5uVJQ3KR4uSmh5ON4XiP8Joxo7umlLkh9V7Ar6zAhhYKcFwX5wc3FUI6Oe1KTgxwrTyb0HF7vupTue0PExobnbbXuOLrCkbi7ljrTZ6bZL)Cpw4PfMDxqvfEAzfqYQLxMA81GXuYioBe8emdlxtPq3hidUFpyLWtUevcgAcm4Ek)tbzgcDz2bNLMBCHW1Zs9YxKDUCz20fsp1vmnZZkYkvjxxkMzmo5dajCQ3KNdIMSV)XiKXKI6EPuhispmP1(m4(ygCFcdU)xqD(PO1Vb3ZWkVxB1NuCoQsiyBLYo4yUbFKd)MnPL6ccX8xq6npWKGQsOJmMM6c2mxA8S0dFjdNDPstfeuhxtD87So(9vh)soPEpBE5FGfoNEgnayeADyaZRzD4REdlx(P)ca3OmZogo0x3EQVkBAQr2OUTQQMtcEAMhQz7zRUOvxJfva1nplZQbSRZHMDn9gkuLkj9IPOkzTsO1Ik4vuQAnfmBtbbn9eS7GnIkPrtitPmlbMTuuYzN2bm0rzPSlQ9dWqImh18tUBMo4XsRdw41QsstvpgtfOb)SNUbkmRHGn3Cq3LVd47mLu(P9UOt3m0Pt3Cw8xPWFja1IZ4aZyv0kTl2X7P3iwMWYDODNlh6OU6ngQm1g3JhRHq9WDR94n6rAMsawhWseiUYhIN6r)u5p5aMsgC1tIQ2q1bN2hjVlv6Hdfyv)o9zZPAH6qPx(9zEZ7NIEuPj)pqvq)JixertKp2YSdAn19UWtmn5zOHhL8h6Ifg5pMhzGTSRMJVLCoZOpYRkcEuqvRRpn5hcVCnYG37jbtMCERti6nsz1zW9rCXLZG73ac5jQfhQQOZun0LYf3Q5O7r7Vz58Pm4(OwmNs3(926PeM(SA9asl8BbSCEBp)Y4P(iUi9e6YtUlGVavIoKO8Y0ZliBx34NpT1Ny3Kmux(wxEA1CIAk5hsuJURLp5lKkVsrD7TWthVYAmpPEpaWcy0RPLM7ak8NmoEu4fa(mfYNnBortEaX7gdfi9l5oTHvNrf5u8TS4VerOqHCMhD5Ytm5J6oPA9wujRyELksvGss1iflG3zdqQ4ljv)UUtvK4cYvuDZJIGlOjN3DkNpHzGyQHjSowKELLWUhJwczrBE(SH0I3RqWiMZUEQ)8gCF1YMjm4(Aaa5Va(XWG7RBW9nWH6lkKwswCgXPr340G7BwY4Rb3FL7wKb33U8wUb33P0u8DR9qPb3LQ9OOb3FDThan4(Ev2iEUstXZBnKzW93qaAdFFoMafo9016WZ8PfMwvqoRYpxp0mFpM9Y18QMcAcZicGw9bZN9NRhAUusZE6TRJNqK16auQhSOO(pNdOsDsAFCToKaf6vOHd(vB4G)Nbdh8LpCabVwhkINh3(GFUEXr8rWU4ADazzq94Czo5p)JzU4eyhnOvp9pe19K3dDpFZgC)ttrcWt(DHc77a)8xjr(OWFEo4N)HQfXpSAr8cvlI)UQfXFz1I4FUAr8CorWZIa(ZZb)8h5Nz0PlwUYVpr8Ca74CbOU99uHZnLgK6E4tjHhK508lJcSFoHC4jeJpLckgFoEYVjVPLMWZK4t)kcWt)3253b)ntViMET83jFR89WFnVkEpobK0dDpTwLm15MQ)9LObzBX492WXoAvuy2G7pH8gm4(t9wf5RP4bpyWOkZiRLxbfZiqaMgAJe5er7jzSXJozY4D3Z9en(KdnsVJny0e2(R4PJmYWJLyYQLWMhau0quRzq9LI5erpvUHYer20)epTDiMU9weOfCwDQL9B2LaqvwEULdCTuywImqxkxzXzj30APOgcV)HYnhvdp5cYI6vwCUL3yTuKvPvX4kVwkalHVRSmSKvFTukJfd00lRybp6ruwawLXeJeFWEN8KJfnrYvFM3fTYgOoHkBrMTPmaLLTu4ktjUdJ19mMPJTQddOPl0y4McgtpMsmZiBme1bQAaVovBWXPva1OdYAdnz(xAPiReubuRTPASeptEBZ2WYyxL0arBqy7fzmBvCANk4mN2DJ(mHQ0BAkF4FsMHhA00tzaaz3JgZld9Wsx5JuYtvrjYMkXlivzHCGw5zk1SwEmCxITQ6zs6w90C3tvquRXwAU1MkXXrOMYQI2c2hlZuvvVH7ywfM9(RRd8JlPxiVQ9rjcc2YEjQnmwSdsx)yTITyHc5vS9n2d2DUCLzsfdUpp8HVGPvsmDhXNNE2NR4q0WoKbJRMZPxspwkECczgDWUVpyLc9MGn2W9pjfgu(5LP2ZxwNSbCVeoT6U8SeDoLdPygNI80n0C70JBOJhsAW9fHrpYJ8Qm4(sgCRyW9LVXfED4UrLyC16tmwFX6QSl30DWcK5dXbhsqdgjLUilWefNEAinlzMV85eXldvZcX5YqfTOZP4nDv3xc73Wz2c1cowBX4b5WDzS(Kjk3dNjc8SR0j2v6LOg9q0BTxJU29XtU85IQNBUHAxk19okUtGS9H0ExhZERMxHyMLXIJnCmt)BWAdjLXlOuYd4NAuPhID7Ksol5TqUm5b5l9Uif3hs2gsQWrYJx5O7GUjKhYpUBtaty37d5tYCv4dpU9LQyLB90lQDIe3ovxBc5TLkCBD0ANDs3ayhF5W6uJCASZCLzZHkzxjzE(cUPE(h44)GF6)wWnTPgDV5EswNre2wn6X(wgIDWnUrZdUXNJCjjCRe1lzRe)CU9Ob3xuB3N1gq9J4OBijivQuip3Zqj69CdBpdXdBxsRR4gC)ddv6(hED2wOTR9BDZwSc96HwROAb8QYJCboYVomtTK6o1lMQ5Wnx6Dei5c8ygtrVQt0pGZTbk5ddZNxiYqa2hdS(JXc7PjxaH23DtBI8Hd39W9YkBNRUJowR3yijMdKjPhyGjtL3Ah)AUQrKIc0YixO(wn7h1J9J2A299bi0lWR4SHGkm8yno0roSlxLyg3lIPUhXVTxsUAUW2wuZaFtBMMwx90mVVoqTowmXX7lQs8CrK9tOsDM1spMmLgx(CzIw(PsHgHh3BlUZuc6OKvIzJzL5cTStwd1((Lfd1mNMDVsJXQuBPQNqsmDwmkycBX4JuXGqPNNZqLFKjPTH(hlwVqgrkE4hl9KbYscgN3EiPLx9z1FoTtzEMQD7W560jnTWSS9lY58f75rpmKhftfhKi7cR0yAQwNRsVkyDpMwXZ(y1D5YYwHvwlZYzRRCzgi(PzX7knjkDDKD57XPXUYeTkNUQkZGRZ(Lhr6XKAvtR9OwPTFl5Ax91XNVkhWIr0CF8ZCD8LRCTnBcKPwnkuRPpn390PKZwSIJ9kEJ7GPOjq)hGOyHgcv2UCkgdNolR4AU5Gn0fDNilPWuSCJ3aUGgohfxZI7mv7CtBh)rdsPU1OhExUxh)VsCC4kUSbC)VQDXd4(F41Kyzhj6MXW8iLU53vrwChzL5vHnRuXWxLP0lkZT4zYSPf7r016SpwY6yxuR9iwxuS9iw2nWu5DlZG9i91O9Ix30vSUZLh33uWYoxLEObPjcRYfyvZfbgqq305Yl)GzJAjpztMhWEBDJTtKBDHTpI8wedRY5HVMhOBVpjiapEqfUxcTnAAyfuJRQdtW4NAEjz7bnNQDxjyEmy8UXsN1SeS50oLec67jV6C03LIg9OuT7QvT2D67LujU7)MdvLC7hyNRslSm5RkRoNlVaOrG672kYfRAcn4X1wGjnxAmys6rqP7ciyOALrtExaH84CduIQ1RzohMt2n6TOsLZdZ9I1YrB0zFVbs2okcOV3Un4(pLih7zmVucqD4Eb2DLY4MVga6ZYD2xF5hLBhI(Lt81HxHPxzGu5I5orUO2vWKkykqfhZGnPNzsx2ZsJQQn1V58OO0lGV1fLwyoeeRr5fRxpknM)1vwPzrU0zqWKkPNLUP9lwJn3Zx59LYm14o7Q5MB2dZ(49DtIfz4vLUIDHwXLEI334kR5cUvVoOAM3mivgbDC0cIcvo7T8XdELLT2SDVtrfYC4(oGPk5XugeMblQsskB9Fv4y7AnDd9jplmEuLocRxwv5QC4(pgBnxThwCw53A7DE4wcxzQoF1o(CM3LjRyFxM0vBDDOQCFMSBsplI9ov6Pm5UxyNQ7g3iO8kSLcytLbd4zVig2PffdI3G7e41qzGvkjynP86fKbD7)K8KNU(wP7Se(08ikXGiaPQEm8DlK)aYheBirn4sSO95zGDXvIxeQSlUstFgJDNqRMogAX27rI1YXcbAYg(UvEdFbTBng(Eng(AWS2n89ADvTrsLxx3DfB4BFqEVn76XW3(Fjvb4wYUgkEYZAW9GwxbluxbN5k3(UwCl1w4Ui(AAtKetPUReXJ1vWEf1P7)MyWr1WLmsQ1rdND3oaK8veTcAibfHGJM3YoSyq07D1GXuMQOURqX8Ixg6LM2eZnDQ85cMFQGdKhh9TSc7iVupQh3izuTKJmQ1Bjf78ECH9OUtA12TwgKqvVItblFf1u3olu8yqLlycj5PkScnKicANv)SYklY(wUIwjThn5P1be)4Yc5m81O14MSyWuZHK8cMa9YC1DBg8mYk4yz(cmlwREJ0WJQOjGIBgxK(UVqP4yw(rNvsoNGzqSr6(Kf1MlyenqeeZErFG2FYtnhoQgrt8CQ3an0yPftHgoJfDHIzenlZHKXxYRGXMgQuZsyKu6YzKfucMap5A6M1emaiMltWjWhtjwhBmf25sbTAv(CZiEjwnPaT6zGLjg(UdZcmHQaU1mbhfA7zYpRHVdQwVRLoMZ(MHzEBVYcZWxiciPXb3CL2a)ss0haTrZRtFaOwM91(K10lGNhNyT8GHgD0h8KpuhM2gVKtLt2BTC7GtFKU2JJzWJ0F8y96HLW3k1s4xNTLW3c1s4AB1dlHhI8lqpqoRILWFdRvlHBW94RKyS(qQHmI2LFrNqTiElvyrCOBH7q(vgFrL3Y44V9sSp(rdqTiDR4ik1C0B10HsC5qbPA1YY0(TCv1zIhp2udnC2XEWgP0FWDoWCsYXG7EEkAw4Mj)l(98DajJyAzqzt3(5VlRX)D4yNLNlXtnm)pIdpOp45(Pv6zc6NWzALEpl6uYzNKjsNzyQBV7uP0eNrgwVZUtT0zNsi)RYPe6f8B476zwY)dYEweC5T0o2I3GlcEht7YW12gGN6V4KlCco)0BaBmfK3g5cqr0n7sQEZKp8s5TFQPGIX8EYMMkQr3VE8UTEGJ)LtKyYlF5VK9h87UiwMkcWy0h1dYf4m4c4dnoEzhGPDZ2xt2F(n4jhEHBKESzvflKI6gVswN5O)yOb(Vl56MbKDRaAW9S8ShYkxVSuSZsHb3)B0o77Roi1Vj2ngOb3)6EB4WTWCmfdUFm1Vum4(jWp)u4N)nB3q5XqxCz(PYjol9z)b4lYNQW5aQ8ZXJ(bZsSppL8mIqQakJZnTGcF4cIk84vRTZvCyaNR3qRRm3zWg180RU)O5McV)d9HaDA3Egrj505ehtwAr7dlc70sAgJe7Wx9zU23WdyW9)zV4wMuX1F4)tnO3pIsU5O(3o9(6gipGKgqsei5b3hgRBDVXBR1oBVfQidFm695F30Nkpq0HDr(fT9rE0x4)u8w3e2GKdOB0))cVrWjpd0AOIn8Xm8D)w7(g6X8OF1J1ZfXe8hZxYMYIxgv03dp6d3j94n67oXnG7QlcrdmxStKnDiXtEQtSEiez4l26IkJHpqyS75)Yizy4BWvHcHHVHwVugm8n8lkccg(gXHkGuvPcy4ButiFhpsD8zQJVac83xDRhKVHVtsX8g(ItX7g(sa)Ke(zS1iaha12qBdFtiz47EHCFFw439W9e)Jvfa)zm8DkhSlc3WxHdZ3DJ8zePNrLbKZkz9QB4cttVl(BTLodF4wbuiduJOY1oygBA1gnttHhWzdFNHHJB5QpC8GroBK7r7e5NQZrFzC8geCCFFr(Ve)xG)lVXfgFT33R(ffk2W3Kg(49cZgpC4oBn86eYcnJvH)lKGAbyB9Qpa70IJpu3TmDF93t3VmG9Nbaw(1aGD37CFqu7CFBmz56Fl)bxHaRPc3A4oBPL1ldwOHSkmyXuul8A4R(WRcNirUhSBD1mreEz86gfbL7To(K1XFOnYck77l)TVsHAb1CBR11TyXqdzvqTykQfQTTR(qTf7BSzt069uS3WrFzu7gfuR9DbYgx5IVN9TwaT4Jk3QADQwps426yDjv89SVvrQyib1cV24vF4105vJ2AMtekSWCVmEDJIuXuK67CdQDO(c7UQi1Vwfi1vNfBRD2sBDUUT80xy3RMLNGuuli7HU6dYorQuhkXHdPOFYX))Vazn8v)bkf3E3BX6ZSJ4rdJe)NfG59S2y(cOMrLyG59)lZ)OBy47IqzesBbNlhh)KRdXKpq4dDKotDiafF4owN449DR1gfdX)hZVUiq8kc9rwfceykQfbI2V6JarVfd3T(jZm(4J06lZtFdfp939guvMjnDLtL5d1whhADRYmPPvtLzif1cU2XvFW1hkC)56VX0N7bYg)LHRBuuz(RSn(V324)N32g5De(36NCLc12wNT3(Hw3MNgAiRMu4qkQfQ9Wx9HANP)HYou3ZM6CJnXlJA3qXKn5g1Tt6v)VSA4vdU(mbShzv2pPwo8Hx)oSb0swT9tcsrTaSDE1hGT1E7w(uD0DMzBFOxgWUrHn7Zwh)xPo(V3gAhV6I(VIjCC4wos7RBHJHgYQXMfsrTqTh5Qpu79EICrg8u5NnRS4lJA3OGAFQTX)r2g)ZUbw44R9zE)xP8ZQd15HpY62pREM3)QSJsqcQfITRR(qS9n3Pk2E)9QFk5ex9Hy3JVD01pNcAFy(ZZ)lUbgXA0CvrSpVb3q8KNUbqfve09jrGAueUTKZjmL0l7PXWoaEkytDNSdnDpc6fcMOy60IIzeZWa4RmzpDNi5KjgRNEIMawSo8EHgbGBxxiDiN1gPdjOwi976QpKUu2XlODO(7vjBFVmV5nkW8DE)8p7P5FQtVbwI6n)OV9RygUQJ2osRR3TDcBiRIe1ykQfQ91DvOFvMsyWC9oXqjYC2xg1UHYWvVNnKW1cDCfcSgUZ1TfRk0rTXPq81cLE0R(qPHspD393Za6NyKmVmkDJcV1F164)i1X)uBOTw1Jh9kM1QA5qh(WTTUTw1JhD1SwfKIAHApgf1266b1Etvb1EtRduRfILEXqGivxax8p2xbUwi3wN9uYDpH(m9LdrUg(2M0Af5EtE8GH(D4SqSai9NYJiwae)VXBFh)YEVqFS1aEL7QA86nugEfw0tXRbw7hgW5tNFgrfbLcisAZuyETXT7FnIBFcYh1dS7(DWU492BP4xe4AHHn1a(b)hQeZUWoOawkkLDS)a5D7G6QZhKNQwmI9(K8OYVdjuiT0KPZxuPaEv905Yt7eq9TqVy0bvCFW)bkzGOg(Edu1OnV2pu3HRupI57Fr9hBDPlm0rQTUWqckh5JnitG)XV6e4NS9ZjFpNix7t0DIxg4))fb(EECh89M2yI5V9b9IpnZCxT2UL5USpWdrz4w3N4basF7dUUGPqDxBykKGAatVJRoHPrLFGbNnsXWjt07ldt))MWuVSu1guy6EEfJhB15nN6qT0XH78ivYC2W3BHINBXW3c2mI3ZR4qFEp5eB47TU(47IL4Q5TZqkQbK(UV6tr5mPJMwi)ud2)qxDAoRdugYvYgNVorU(FjCOg29E4EV9ZxlJwzEQgaC0o3GPbm(cKmzbHCIkfQYPB4DWavKp(lgMK)hVPvHjjKGAPf7b3qdPK8csPEQHUVSrsFQjeUk0tP2ZqpXGaQA)LcJCHXQbIc(3VH9hmru2p0muoI7MCH6BzndS2HBwI)78vGN(Vwae0qUb(1miQAhjOpSlqZUPxwTYfeNgVQJ5ldcnuG5TIAp(B8J5jEYgkDX9WDQwPaQOQ7X6QdeVJsPp6abBmgusHiBncwIuG2ZOUJbYNBUGtKxltxb7rsqVGSUi7CbHYmZU)7qLxLSFiY(FWFjAi2VNJ1FmSNOwVZ1)NtNj6syDgShmdKTA479e3uXzdFVly09Dt7tpRAJopwmM395HCUF(PF)0TEg8QjMED3sMVmYbDF1h5aHJ02zpx8XhAGwICvi5GEVWNCJc5GDUkKdawRxvsvabivqlarm((BFtRzQag(wKI6yGDdF)QoqBdFVx3yAdFpgdhB479rLE(9tZOHVh3bRI18ZA47d4nims11CT6pOBxbaHVtFRnfxLCP4kddM9SJlLp04Nl)e3BTwNrXNMyZtSHdBUWR2t9NvVPkUaCBRzDZpcJKMTKlXZEW7(UpShziS7m8tMR8haV)DoYpmYTbLAvFc8yAF7NscbPFaKrasM4kgt6ixBNZ5fDKdSBR7l3dvYd926K8Y4oKxab49vpKxwvX5bHhQTaSO(GCp8qOdPRkf64jDrE5gQg5LRDxlyEN0tjoWOU8PCiWa5(IyA8G8Ys9jNlxWrLKfYztM5RbKz49ImdtCbzL05kMree8qlROoF1P9avjRaFcYYSgDxAt0D8bhPN7HAkG7Lh9kn6JIawyGqiIAYGWmP1n4sI7expS3Ka5Sk51e7vwpDEffrivzWwxhGii9UMebX5LV50Hl5BT5iEc(6DqFlRrYJaptk1XqlCCIp)Ke7gV42JqFStPovhED0lrV6VjheONHuTO3u(P6zSejhzOxc3w(VsYOZNCKrJhR)bsAj3bEJ5d0121En89BQUd2l9sjVwPZ6RjNXHPWxMx9MckqFUEJlMfcv39ltg9LDip9b8TK3Wd7xXIgk5cp39dGc7zrXjfSaCLIPYdnpxjG(DRxkLsFlIMSjyMcBC9kuqW1BVK7MT73FiwJ38vBXjNnZ(yZSNWj8vbQkX1mM1Ykn6yL5BvsrR3BixJpNgd8mELj8vdXoZyUMS)txrgpJNV5HS8xXJpeBE700)CgRI3(7WaMRh6r6KHSIUOwHgljDUhtdvThildFBfF6n7OZejdhQ9KN6SzWRRFxV0T2xz)mU0zVv8c7)b2SHVxPHVpQ9L4pYByFBganFo6DIV5f2VRRgF2f5Fo)492pEp(J3X)SN0w867NEt(7h5RFnbiTGVbgKBK8wI07itmCPVTTaL8(mV4(7Y1f6)R7f9JA7H8JVAVpc5nDr3yiFl0n5TAW9lbY)yWTldUDZEWlWE3JsEh4qaB4i1xN8U4iVBYIK3d5xL8Ejpg59Xyz(bG)WrnEUb3JB4RFYhsI0YVfJ94wCmsnYE0W3fSVu5BV8xdvlRF9(a2IUSQ13PU6cUPnTVbo(F7DDx8BAtBZpZYyUF1xxufFOpWxnfG7O7IXVDUg44)(7DVh)Yx(5lx4Dkd8po5cBP0Q)V7B(nVmK6bo(N9XF8p9LV8JuU11Qi)(lRETB2e2hQw(2CP17xB8XHMRVbo(30Qb8x3wB39M20oQw(VUQnS9X)4q7(hA3tQw(3U78Zz3U9xT0FdLwFF9h)X)d30M6eY4pGoq9TQw(UPsZ3p4ihbAN)JWWZ76D9)ZLV8kdC8)Xp9N(BF5l)FwT8)APpmUtvJN1c67KRRRUKNhXbg(27S(ALrK3Evs1E2RyVxmyk86vVI(Y47kIzeYvK9EgwcDPthvP40npQvD1CFf1MJ(e9JFOSN8OYt7Gfve0OFLMdZNbgAaRsohsqmhK680hdVgS)2QKROkIAzzTo2hxL03hYoJ1zCXyRAPoMIUaWgGv(wFzv7hkcMDbfHvjTXfYs5u0a(HvlTadP0oJS0VwXa75l5fnJ(2jaSw0PVFtL88M9R7iUXA99ndFm0NC0rMiA8jhB0E7oz0v5bjRwp)Eyny(ILr7cngY6bJ07NanZha)KDhV)OjNSNb6E4(J27lH6Vbt5ESAeiJ)gxpTO(gbey8krdIjN16V9Kiz3jhR83iotP0OcgzlF2R3(XlZEyWU(pVB51wZpMIy5xRw(A6Pa2(bbT0xcn14Ens3uqpxr0uWkwK2fRZ2fRN2fTBsIErqBLcbhIbn84Pvdi(odGtux2bMyInGslFXcEap8CcWSVLqE2XInjmeoin3uXnllSgdXkLcqCcAzAWEwU8YI1a8kXMYgcJz7mrS7fkCyiAKXShCSuhe7R(8XihuAVnMtVnK7x1YgzT1y69KRybqdV7rCUErlCJVaw69KxbOtNZmSqngYQr02qJ0BS(IbZe4qtuN5iwJRNbhlzYYJSkTrqpXqU0t0rdru3qCWeFQLlGQg6wzW)eM0D4ZXMLzXwo)mIAWiy6CYQ8KpRevpVfafgFR(RFwU6NJBr6deesWL94UXzW9Ram0F7Wppk8Z7Wp5Pd3AlTaA6zW9o9ZsDFqlZh5voVMyobu1huLqtl4dXkHAgAWbsEUTf2Htfz47tv1AWW3FGzfWmhNHV)qjdFpJpMUMg((02fz1my3Og((VBW1Hev7tQb7GbHGNIAfn8xFXzCBkTv4Pwt7Rq(QMMt7oWxjQBUcdtPmPG972KKB7EHwOIAEkdU3V6T4r(WhCzZVrTELYTDBg(U(71)HAUD1Gvl9tNpZK4lNhVHVyCE8YsPu5JwfAzn1xtnB4toTWSB3J0uGvPoV)vE0rkushrnu1sqM5ueMwo9KfK0e1LYNlZCQ3OhP10iF)yp7B0ckNywH0Zn5u5YdlK8ygrDsn89PBsueViPfLZr(wKVn5h5N8VMI9WA55SOtEm89fiF3hMAOWxGJ8DE1BsDVvK(0HTTgjZuPEoax(GNxDPkEiWq7U(C3KB7yAW9wCBitddF)5EmmR8IyyoVDRNUS2W3Fuvx6A47ICudaBT20W3Zsx8z47)nSYcWGFMTB47pbkJ)uiOp7Cg((CKFSHV)miIpposAn2Vye5SSHFdFFX40jVshLn8rSgmn8bkK5BLvFKaZWxgQYVcaVLg1FhG65)NKlBWTjo2(FXbkV6d(5AaDtEQDUZB9Yx(NmWXxU(6bDHcoWX)e7AxqiphOnzBT15M20TqAD3gCx)JLsKk0SNQzgNYR0tnbxwGkxpL9EnvzBEztHM9urRitbkuutvPwsZwQ3AQA5ItBPVqjAagzAqg8AQkEe8zJCv0woGpZTk8ZzVDfZ(kYzjzKJi62ksvnfOCECzRTAtvpDLOKu1tMJwrvpn2Acv9K4w9NA1Myk8u9uyPMtnsrjk3Ws35deWEm25LU3Eq2rCt0uB2chGVa3HkZEF2Qn3K5hZFwrSGWmB3gO5KAIXqEA0puedx5UcR)rZU1RE)rzFfFQQc(gnFEIRm10OlpwrptAvkyCKcfdJgOJ1bGyS)sLLLBRiuJKz2688H63lb4XmzjiAtUMIy2KO6ZrWyRt9vXWQRzq3ZyoZwnvwNkKhzhy5pA5LWqcZ(IRquf1sdc7Bvq2Ljmy3cvi0gzTY70oQqbpqqqyrCcsroxGQ(K6x2mCjJ42XB(ow7UvuzIkKVa0sDACvMcQnFSM4xhZRaQ0EICTRCyaxGi74yMyV0P8YJ8ObPO(gR8Hk3C8OLkFCTDV(2ROn9lNK2pg8Gu7P94P82AW0JYWCtmOlDQisRHyx578MlWCOJ58QFBpMBzVGshcyQtjWt2MZ(KxN9hn4Uw4NTYCfGFt8H1fTI)Pj3iPDdUTBWDDmjiU9hL5BSCB7OZBQcK(EXOg44F1B5P)fV8LFkdU6ixM8NYro6EFEdUVkp5U2fjfW873dk(VgVb3Fb8xddUVUb33a(0tccpyW9n)atrtZtA47RJ6QD9SKV8fDB2pdUNQMIiyxe)LUlIiiptwwRU0b2z9VYDwx0MjjMF3IdyN()c3PpfJBjMyVedYotgUZuCk7tZ8uH8r255B4opZBXpL1VQuEi7S9TkzOazWw(Oy5YXyN1VDjzf58YYA1fWXoRFt3zDzxgyKvcvx6lqa1om4(BqDs)(C4GtpOIVgC7CxKtU8OHKNONPsKov6ue)(XNh9m7IKy3xkX4r6k44YzbIJ09I11Zvo5(FPT)ROOZ2pu571W3FpU5RhKRYxdB8eoEItPmt)JLOXZEeC7Ly7xM7x8Azo79l7b2kUDzBNupUvx7LUhyiA7Taah2oLnSFCZYWDkd0eLTrz4wMzUrzA09kRLaL(CxJn2Nuc3)UATjz42BTw3Lm8n721M4HBTh9nUokBxYGrGtdTE05GWTi7Tlv2UKn6M7mCZhU9QSxzVFYJBUFzOBLGBuwaY)nj6wBSW2D8wd6UCGhmHnZVf0DnuRdnisgqA78kAY47FopZdXw4w(Y49V17M)P2g)dZVZ9XBFl69i1X)L4zh)Pdvh)7eFRM(I8)Y4BB655)c89cjJETD992g)ViE32MCF4PK69WZCjeQVGeaDmd3(Zr84J0)yrP(TX)e1LocqMX1y9S2BctP(znE8i2JPNueb0ir0p1g6N9fVn0hz0KXgzyVmtRSsgXzbMgT2uWowxgHLwawgETrqSJMqzpEXA7v3cFAA9y2(Y7S9(wvz5TsA66Aa5mIRMfFXcMTh6YUeIiwb0VwfPKeOBbVEJHwxJe0IU5shpyHr5mBLak791ql9fXqakkl1tfSeMRcXz9Aq3RwGRn8pOLbvVDtBEhD4KrJhB4(NCIrIpiAZBQHUNCSytorSE7pAsh7zJMo9tYZ2Cz0CpaX5WKF88iuexgZtDCr2toUvsyoNi6ZrarMNU(wn8DVsKOXP8bS3cj6XtaLxFefEkvh8edGoN4NX2ORCoUIiiuHPLwRKIoqZyTysuM1qXCA47zu)ZTaEn62RysJQgE(YcWwds6Rj(PzBbAJHBcvdOhmOgT2nKqHoJR8Q1uWSnfmfdqI)3cl7uG5vMso7PBGwWnWeJ1v2s3m850nNf)vk7PtZPEZKHtSbcG0YdGM41XQUvmoDYLJmQE)5cj3D(yZr(ZCzxxolhxUMw0LAZxhRxSLDB3ngpw)Je3CHZKX6fA5h6qD0sTv9gX41qHUAPoJ369wMgJvu2nf0e5wgM1lvJzsFdkLX(GLYXmH5rvMXpeWtDIO)nGNku5rUkx)41N6rmYuERFenUQQGKzSvxdPsNcQupf3tdRd1KwF60WA1Sv(EosnHCMSIfIPmv(sq3NZoyOX0dqSJLou24SIAndzeDdYXuzCoyroUSErHCYpKawj0sK2GlBjFiNwlmd4QEqD6D(kD(wH2lWLrudyXQMe2r0mGmYuUXHmhcCkj37wNxdcEXiSAJeyhQ8rnQRMfeAFjXfTSsavh1T)7z2OsNxlFXcYkInFonbNJdsJLA)dQ8jmsHUgpkRckJvOt71cD1OCWx3rlRa6RyoR8x2O9uWS5qcZgQIcfAlqjDSQxq3rWwPfwlEM3YQIXlbYBJCShwMtwmxMgF92INj3uWsOw0KTvcQSTh88H8K)o83qvBUFefxcqzTiNY8fusc3QvbOvXwv0vcXcDNtvsOXwRrXHlonlo6AfgfwAHb6O9ISIAPQvKNfRjByKbjnykYTh7VIGwNV1OnV5GEXCUuuQRYSeSgfFB6ZOPe0Ww(4GCnIZYQcNSHyM49hPXqHkDVZTSZIb3BI(L6yF5nt)Ih73IBfTWtbXAC3xy(PmiDwyY3MTbdxRe1eo)9OXCO7CBtBMURdOzCOEVVJPCwrxvq7S9kQlayyrj0OomR3iDtGYRmxGKQ8nEMnEBsyZ9)V")
end

-------------------------------------------------------------------------------
-- Ignore List
-------------------------------------------------------------------------------

function M:GetIgnoreList()
	local ignoreList = {}

	for target in next, self.config.blacklist do
		if self:IsIgnored(target) then
			ignoreList[target] = target
		end
	end

	return ignoreList
end

function M:Ignore(target)
	if not self:IsIgnored(target) then
		self.config.blacklist[target] = true
		table.wipe(self.isAncestorIgnored)
		self:ReregisterRegions()
	end
end

function M:IsIgnored(target)
	return self.config.blacklist[target] == true
end

function M:IsAncestorIgnored(target)
	local isOptionsOpen = WeakAuras.IsOptionsOpen()

	if self.isOptionsOpen ~= isOptionsOpen then
		self.isOptionsOpen = isOptionsOpen
		table.wipe(self.isAncestorIgnored)
	end

	if self.isAncestorIgnored[target] ~= nil then
		return self.isAncestorIgnored[target]
	end

	local isIgnored = self:IsIgnored(target)

	if not isIgnored then
		local data = WeakAuras.GetData(target)

		if data and data.parent then
			isIgnored = self:IsAncestorIgnored(data.parent)
		end
	end

	if not self.isOptionsOpen then
		self.isAncestorIgnored[target] = isIgnored
	end

	return isIgnored
end

function M:Unignore(target)
	if self:IsIgnored(target) then
		self.config.blacklist[target] = nil
		table.wipe(self.isAncestorIgnored)
		self:ReregisterRegions()
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "WeakAuras" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

function M:LOADING_SCREEN_DISABLED(...)
	self:UnregisterEvent("LOADING_SCREEN_DISABLED")
	self:ScheduleTimer("PrintImportReminder", 10)
end

function M:SIXUI_CLUTTER_STATE_CHANGED(...)
	WeakAuras.ScanEvents(...)
end

function M:SIXUI_LAYOUT_CHANGED(...)
	WeakAuras.ScanEvents(...)
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:Border_SetBorderColor(region)
	local border = B:GetBorder(region:GetParent())
	if (not border) then return end
	border:SetColor(self.GetColorForBorder(region))
end

function M:Icon_SetVertexColor(region, r, g, b, a)
	self.hooks[region].SetVertexColor(region, r, g, b,a)
	local border = B:GetBorder(region:GetParent())
	if (not border) then return end
	border[a == 0 and "Hide" or "Show"](border)
end

function M:Region_OnHide(region)
	self:ResetRegion(region)
end

function M:Region_OnShow(region)
	self:StyleRegion(region)
end

function M:WeakAuras_GetRegion(...)
	local region = self.hooks[WeakAuras].GetRegion(...)

	if region and not self.regionRegistry[region] then
		self.regionRegistry[region] = true
		self:RegisterRegion(region)
	end

	return region
end
